C
c     Description
c
c     Solve the Fabry-Perot equation (relating wavelength, Z_fine, and
c     distance from the optical axis) for unknown parameters, given
c     information from a sausage cube about the Z_fine values of lamp
c     emission lines (and possibly their relative order numbers and
c     optical axis radii).
c
c     Authors
c
c     Ben Weiner and David Rupke
c
c     History
c
c     ???      BW    created
c     08jan14  DSNR  modified output
c     08jan15  DSNR  interactive --> command-line options
c     08feb19  DSNR  fixed bug for iford=2,np=2
c                    improved fsr_lambda computation (n->n+1)
c                    added fsr_z computation
c     08feb23  DSNR  added plotting to screen
c                    more comments, plotting commands re-arranged
c
c
c     Physics
c
c     The equations to be solved are 
c
c     D = e + g*z
c     wave = 2 * D/N
c     FSR = wave/(N+1)
c
c     where D is plate spacing and is assumed linear in z.
c     Combining the first two equations,
c
c     wave = 2e/N + 2g/N * z
c          = a + bz
c
c     where  a = 2e/N and b = 2g/N.
c
c     If every line is in the same order I can just solve for a and b
c     and have no information about N or FSR.  If lines are in different
c     orders, I can solve for N0.  The user should know the delta-order
c     for a line, i.e.  N = N0 + dN, where dN is 0, 1, 2 or something,
c     and the user can supply dN but we need to find N0.  We then solve
c
c     wave = 2/(N0 + dN) * (e + g*z)
c
c
c     Generalizing this to include radius, 
c
c     wave(r) = wave(0) / sqrt(1+r^2/f^2)
c
c     so r is another column of data and f is another parameter.
c
c     For only one order:
c
c     wave = (a + bz) * (1 + r**2/f**2)**-0.5
c
c     For multiple orders:
c     
c     wave = 2/(N0 + dN) * (e + g*z) * (1 + r**2/f**2)**-0.5
c

      program fpsolve

C=============
C Definitions
C=============

      implicit none

      integer maxlines,nparams,maxord
c     Maximum number of lines in fit
      parameter(MAXLINES=200)
c     Maximum number of interference orders
      parameter(MAXORD=12)
c     Number of fit parameters
      parameter(NPARAMS=4)
      
      real werr(MAXLINES)
      real xpl(2,MAXORD),ypl(2,MAXORD),xpls(2),ypls(2)
      real wpred(MAXLINES),wcenpred(MAXLINES),wresid(MAXLINES)
      real aorder(MAXORD),border(MAXORD),dordval(MAXORD),fsrord(MAXORD)
      real xindex(MAXLINES)
      real pars(NPARAMS),errpars(NPARAMS),sumsqerr
      real chisq,covar(NPARAMS,NPARAMS),alpha(NPARAMS,NPARAMS)
      real a,b,e,g,order0,order,fl,fsr,fsrz,q
      real da,db,de,dg,dorder0,dorder,dfl,dfsr,dfsrz,siga,sigb
      real wmin,wmax,wavg,wmarg,wrmin,wrmax,wrmarg,zmin,zmax,zmarg
      real xplot1,xplot2,yplot1,yplot2,vpx1,vpx2,vpy1,vpy2
      real pxlg1,pxlg2,pygrow,pylg1,pylg2
      real xlg1,xlg2,ylg1,ylg2,xlg1lab,xlg2lab,ylg1lab,ylg2lab
      real vpy2new,yplot2new

      integer i,j,iford,ilen,ios,ifit(NPARAMS),ifconv
      integer nargs,nordval,np,npars,pgstat,psplot

      character*80 arg,fname,psfile,datfile

      external waveordfunc,waveradfunc,waveordradfunc

      real wave(MAXLINES),z(MAXLINES),rad(MAXLINES),dord(MAXLINES)
      common /linedata/ wave,z,dord,rad

c     PGPLOT subroutine declaration.
      integer pgbeg

C=========================
C Set initial parameters 
C=========================

      npars = NPARAMS

C     Get command-line arguments.
      nargs = iargc()
      if (nargs .lt. 2) then
         write (*,'(/,a,/,15(/,a),/)')
     $        "Usage: fpsolve (fit type) (input file)"
     $        ,"Equation to be solved:"
     $        ,"  wave =     (A + BZ)*[1 + (R/F)^2]^-0.5"
     $        ,"       = 2/N*(E + GZ)*(1 + (R/F)^2]^-0.5"
     $        ,"  where Z = etalon fine spacing"
     $        ,"        R = radius in pixels"
     $        ,"        F = camera focal length in pixels"
     $        ,"        N = interference order"
     $        ,"        A = 2 E / N"
     $        ,"        B = 2 G / N"
     $        ,"Fit types:"
     $        ,"  (Type) [parameters fit] {input data}"
     $        ,"  (0)    [A, B]           {wave, Z}"
     $        ,"  (1)    [E, G, N]        {wave, Z, relative N}"
     $        ,"  (2)    [A, B, F]        {wave, Z, R}"
     $        ,"  (3)    [E, G, N, F]     {wave, Z, relative N, R}"
         stop
      end if
      call getarg(1,arg)
      read (arg,*) iford
      call getarg(2,arg)
      read (arg,*) fname
      
 20   open(2,file=fname,status='old',iostat=ios)
      if (ios .ne. 0) then
         write(*,'(2(a))')
     $        "FPSOLVE: Error opening data file: ",fname
         stop
      end if
      np = 1 
 30   continue
      if (iford .eq. 0) then
         read(2,*,err=40,end=40) wave(np),z(np)
      else if (iford .eq. 1) then
         read(2,*,err=40,end=40) wave(np),z(np),dord(np)
         rad(np) = 0.0
      else if (iford .eq. 2) then
         read(2,*,err=40,end=40) wave(np),z(np),rad(np)
         dord(np) = 0.0
      else if (iford .eq. 3) then
         read(2,*,err=40,end=40) wave(np),z(np),dord(np),rad(np)
      end if
      xindex(np) = real(np)
      np = np + 1
      go to 30
 40   continue
      np = np-1
      close(2)

C==========
C Get data
C==========

c     Open output text file
 50   ilen = index(fname,' ')-1
      datfile = fname(1:ilen)//'.dat'
      open (3,file=datfile,status='unknown')

      write (*,'(a,i3.0,a)') "Read in ",np," points."
      write (3,'(a,i3.0,a)') "Read in ",np," points."

      if (iford .eq. 1 .or. iford .eq. 3) then
c     Find the number of distinct orders
         dordval(1) = dord(1)
         nordval = 1
         do i=2,np
c     For each wavelength, check if we already got this order
            j=1
 90         continue
            if (dord(i) .ne. dordval(j)) then
c     Doesn't match               
               j=j+1
               if (j .gt. nordval) then
c     Hit end
                  nordval = nordval+1
                  dordval(nordval) = dord(i)
               else
c     Test against next value
                  go to 90
               end if
            end if
         end do
      else 
         nordval = 1
         dord(1) = 0.0
         dordval(1) = 0.0
      end if
c
c     Get limits in wavelength and Z_fine
c
      wmin = 1.e10
      wmax =-1.e10
      zmin = 1.e10
      zmax =-1.e10
      do i=1,np
         wmin = min(wmin,wave(i))
         wmax = max(wmax,wave(i))
         zmin = min(zmin,z(i))
         zmax = max(zmax,z(i))
         werr(i) = 0.1
      end do
      wmarg = (wmax-wmin)/10.
      zmarg = (zmax-zmin)/10.

C=========
C Fitting
C=========
c
c     Fitting
c
c     If not fitting for order, can just do a linear fit
c
      if (iford .eq. 0) then

         call fit(z,wave,np,werr,0,a,b,siga,sigb,chisq,q)
         if (np.gt.2) then
            chisq = chisq / real(np-2)
         else
            chisq = 0.0
            siga = 0.0
            sigb = 0.0
         endif
         sumsqerr = 0.0
         do i=1,np
            wpred(i) = a + b*z(i)
            wresid(i) = wave(i)-wpred(i)
            sumsqerr = sumsqerr + (wresid(i))**2
         end do
         xpl(1,1) = zmin-zmarg/2
         xpl(2,1) = zmax+zmarg/2
         ypl(1,1) = a + b*xpl(1,1)
         ypl(2,1) = a + b*xpl(2,1)
         write(*,'(a,2(a9))') '   ','a(A)' ,'b(A/[z])'
         write(3,'(a,2(a9))') '   ','a(A)' ,'b(A/[z])'
         write(*,'(a,f9.2,f9.4)') '   ',a,b 
         write(*,'(a,f9.2,f9.4)') '+/-',siga,sigb
         write(3,'(a,f9.2,f9.4)') '   ',a,b 
         write(3,'(a,f9.2,f9.4)') '+/-',siga,sigb
c
      else if (iford .ge. 1) then
c
c     Fitting with order or radius is non-linear.
c
c     In principle I could fit the same function and just keep the
c     radius/order fixed.
c     
c     Initial guesses.  (DSNR: Result not very sensitive to initial
c     guesses.)  In order: e, g, lowest order, and focal length.
c
         pars(1) = (wmin+wmax)/2.
         pars(2) = 0.3
         pars(3) = 25.
         pars(4) = 355.0 * 66.67
c     Fit / hold fixed array.  1 = fit, 0 = hold fixed.
         ifit(1) = 1
         ifit(2) = 1
c     Fit e, g, and order.
         if (iford .eq. 1) then
            ifit(3) = 1
            ifit(4) = 0
c     Rescale so the parameter guesses are e,g rather than a,b.
            pars(1) = pars(1) * pars(3)/2.0
            pars(2) = pars(2) * pars(3)/2.0
            call mrqfit(xindex,wave,werr,np,pars,ifit,npars,
     $           covar,alpha,npars,chisq,waveordfunc,ifconv)
c     Fit a, b, and radius.
         else if (iford .eq. 2) then
            ifit(3) = 0
            ifit(4) = 1
            call mrqfit(xindex,wave,werr,np,pars,ifit,npars,
     $           covar,alpha,npars,chisq,waveradfunc,ifconv)
c     Fit e, g, order, and radius.
         else if (iford .eq. 3) then
            ifit(3) = 1
            ifit(4) = 1
c     Rescale so the parameter guesses are e,g rather than a,b.
            pars(1) = pars(1) * pars(3)/2.0
            pars(2) = pars(2) * pars(3)/2.0
            call mrqfit(xindex,wave,werr,np,pars,ifit,npars,
     $           covar,alpha,npars,chisq,waveordradfunc,ifconv)
         end if
c     Reduced chi squared.
         chisq = chisq / real(np-npars)
c     Set errors based on covariance matrix.
         do i=1,npars
            errpars(i) = sqrt(covar(i,i))
c     Write covariance matrix to output
c            write(*,'(4(f9.4))') covar(i,1)/sqrt(covar(1,1))
c     $           /sqrt(covar(i,i)),covar(i,2)/sqrt(covar(2,2))
c     $           /sqrt(covar(i,i)),covar(i,3)/sqrt(covar(3,3))
c     $           /sqrt(covar(i,i)),covar(i,4)/sqrt(covar(4,4))
c     $           /sqrt(covar(i,i))
         end do
         e = pars(1)
         g = pars(2)
         order0 = pars(3)
         fl = pars(4)
         de = errpars(1)
         dg = errpars(2)
         dorder0 = errpars(3)
         dfl = errpars(4)
c     Predictions at each point
         sumsqerr = 0.0
         do i=1,np
            order = order0 + dord(i)
            if (iford .eq. 2) then
               a = e
               b = g
            else
               a = e *2.0/order
               b = g *2.0/order
            end if
            wcenpred(i) = a + b*z(i)
            wpred(i) = wcenpred(i) / sqrt(1.0 + rad(i)**2/fl**2)
            wresid(i) = wave(i)-wpred(i)
            sumsqerr = sumsqerr + (wresid(i))**2
         end do
c     Go through the orders that actually were used
         wavg = (wmax+wmin)/2.0
         if (iford.gt.1) then
            write(*,'(a,f7.0,a,f4.0)') "F(pix) = ",fl," +/- ",dfl
            write(3,'(a,f7.0,a,f4.0)') "F(pix) = ",fl," +/- ",dfl
         end if
         if (iford.eq.0.OR.iford.eq.2) then
            write(*,'(a,2(a9))') '   ','a(A)' ,'b(A/[z])'
            write(3,'(a,2(a9))') '   ','a(A)' ,'b(A/[z])'
         else if (iford.eq.1.OR.iford.eq.3) then
            write(*,'(a,a7,2(a9),2(a10))') '   ' ,'order','a(A)' ,
     $           'b(A/[z])' ,'FSR(A)*','FSR([z])*'
            write(3,'(a,a7,2(a9),2(a10))') '   ' ,'order','a(A)' ,
     $           'b(A/[z])' ,'FSR(A)*','FSR([z])*'
         end if
         do j=1,nordval
c            order = order0 + dord(i)
            order = order0 + dordval(j)
            dorder = dorder0
            if (iford .eq. 2) then
               a = e
               b = g
               da = de
               db = dg
            else
               a = e * 2.0/order
               b = g * 2.0/order
               da = a * sqrt( (de/e)**2 + (dorder/order)**2 )
               db = b * sqrt( (dg/g)**2 + (dorder/order)**2 )
            end if
            fsr = wavg / (order+1)
            dfsr = fsr*(dorder/order)
            fsrz = wavg / 2. / g
            dfsrz = fsrz*(dg/g)
c            fsrz = fsr/b
c            dfsrz = fsrz*sqrt((dfsr/fsr)**2+(db/b)**2)
            if (iford.eq.0.OR.iford.eq.2) then
               write(*,'(a,f9.2,f9.4)') '   ',a,b 
               write(*,'(a,f9.2,f9.4)') '+/-',da,db
               write(3,'(a,f9.2,f9.4)') '   ',a,b 
               write(3,'(a,f9.2,f9.4)') '+/-',da,db
            else if (iford.eq.1.OR.iford.eq.3) then
               write(*,'(a,f7.3,f9.2,f9.5,f10.1,f10.1)') '   ',order,a,b
     $              ,fsr,fsrz 
               write(*,'(a,f7.3,f9.2,f9.5,f10.1,f10.1)') '+/-',dorder,da
     $              ,db,dfsr,dfsrz
               write(3,'(a,f7.3,f9.2,f9.5,f10.1,f10.1)') '   ',order,a,b
     $              ,fsr,fsrz
               write(3,'(a,f7.3,f9.2,f9.5,f10.1,f10.1)') '+/-',dorder,da
     $              ,db,dfsr,dfsrz
            end if
            xpl(1,j) = zmin-zmarg/2
            xpl(2,j) = zmax+zmarg/2
            ypl(1,j) = a + b*xpl(1,j)
            ypl(2,j) = a + b*xpl(2,j)
         end do
      end if

      write(*,'(a,f5.0,a)') '* For a (average input) wavelength of '
     $     ,wavg,' A.'
      write(3,'(a,f5.0,a)') '* For a (average input) wavelength of '
     $     ,wavg,' A.'

c     Reduced chi-squared and RMS error
      write(*,'(a,f6.2)') 'Reduced chi^2 = ',chisq
      write(*,'(a,f6.2)') 'RMS error in wavelength = ',sqrt(sumsqerr/np)
      write(3,'(a,f6.2)') 'Reduced chi^2 = ',chisq
      write(3,'(a,f6.2)') 'RMS error in wavelength = ',sqrt(sumsqerr/np)
c
      write(*,'("Writing solution to output file: ",a30,/)') datfile
      close(3)
c
c     Limits of wavelength residuals
c
      wrmin=0.0
      wrmax=0.0
      do i=1,np
         wrmin = min(wrmin,wresid(i))
         wrmax = max(wrmax,wresid(i))
      end do
      if (np.eq.2) then
         wrmin = -0.01
         wrmax = 0.01
      end if
      wrmarg = 0.1 * (wrmax-wrmin)

C==============
C Plot results
C==============

c
c     Plot to screen first.
c
      psplot = 0
C     Look for proper screen device
      pgstat = pgbeg(0,'/XS',1,1)
      if (pgstat.ne.1) then
         write(*,*) "/XS device unavailable; trying /AQT."
         pgstat = pgbeg(0,'/AQT',1,1)
         if (pgstat.ne.1) then
            write(*,*) "/AQT device unavailable; querying."
            pgstat = pgbeg(0,'?',1,1)
         end if
      end if
c     Set up plot
 500  call pgask(.true.)
      call pgscf(2)
      call pgsch(1.4)
c     Plot each actual wavelength and measured Z_fine
      xplot1 = zmin-zmarg
      xplot2 = zmax+zmarg
      yplot1 = wmin-wmarg
      yplot2 = wmax+wmarg
      call pgenv(xplot1,xplot2,yplot1,yplot2,0,1)
      call pglabel("Z\dfine\u","Wavelength","")
      call pgsch(1.75)
      call pgpoint(np,z,wave,17)
c     Plot lines of constant order for R = 0
      if (iford.eq.0) then
         xpls(:)=xpl(:,1)
         ypls(:)=ypl(:,1)
         call pgline(2,xpls,ypls)
      else
         do i=1,nordval
            xpls(:)=xpl(:,i)
            ypls(:)=ypl(:,i)
            call pgline(2,xpls,ypls)
         end do
      end if
c     Plot predicted wavelength for the given Z_fine and R
      call pgsci(2)
      call pgpoint(np,z,wpred,5)
c     Plot predicted wavelength for given Z_fine, assuming R = 0
      if (iford.ne.0) then
         call pgsci(3)
         call pgpoint(np,z,wcenpred,7)
      end if
      call pgsci(1)
c     Plot legend
      call pgqvp(0,vpx1,vpx2,vpy1,vpy2)
      pxlg1 = 0.00
      pxlg2 = 0.50
      pygrow= 0.15
      pylg1 = 0.10
      pylg2 = 0.05
      vpy2new = vpy2+pygrow*abs(vpy2-vpy1)
      yplot2new = yplot2+pygrow*abs(yplot2-yplot1)
      xlg1 = xplot1+pxlg1*abs(xplot2-xplot1)
      xlg2 = xplot1+pxlg2*abs(xplot2-xplot1)
      ylg1 = yplot2+pylg1*abs(yplot2-yplot1)
      ylg2 = yplot2+pylg2*abs(yplot2-yplot1)
      xlg1lab = xplot1+(pxlg1+0.02)*abs(xplot2-xplot1)
      xlg2lab = xplot1+(pxlg2+0.02)*abs(xplot2-xplot1)
      ylg1lab = yplot2+(pylg1-0.01)*abs(yplot2-yplot1)
      ylg2lab = yplot2+(pylg2-0.01)*abs(yplot2-yplot1)
      call pgsvp(vpx1,vpx2,vpy1,vpy2new)
      call pgswin(xplot1,xplot2,yplot1,yplot2new)
      call pgsch(2.)
      call pgpt1(xlg1,ylg1,17)
      call pgpt1(xlg1,ylg2,45)
      call pgsci(3)
      call pgpt1(xlg2,ylg1,7)
      call pgsci(2)
      call pgpt1(xlg2,ylg2,5)
      call pgsci(1)
      call pgsch(1.)
      call pgtext(xlg1lab,ylg1lab,'Measured Z\dfine\u and \gl')
      call pgtext(xlg1lab,ylg2lab,'Sol.''n (for constant N and R=0)')
      call pgtext(xlg2lab,ylg1lab
     $     ,'Pred. \gl, for the meas. Z\dfine\u and R=0')
      call pgtext(xlg2lab,ylg2lab
     $     ,'Pred. \gl, for the meas. Z\dfine\u and R')
c
c     Plot wavelength residuals as a function of Z_fine
c
      call pgsch(1.4)
      xplot1 = zmin-zmarg
      xplot2 = zmax+zmarg
      yplot1 = wrmin-wrmarg
      yplot2 = wrmax+wrmarg
      call pgenv(xplot1,xplot2,yplot1,yplot2,0,1)
      call pglabel("Z\dfine\u"
     $     ,"Wavelength Residual (actual - predicted)","")
      call pgsch(1.75)
      call pgpoint(np,z,wresid,5)
      call pgend()
c
c     Plot results to file
c
      if (psplot .eq. 0) then
C     Look for proper plot device
         ilen = index(fname,' ')-1
         psfile = fname(1:ilen)//'.ps'
         pgstat = pgbeg(0,psfile//'/PS',1,1)
         if (pgstat.ne.1) then
            write(*,*)
     $           "/PS device unavailable; not producing postscript."
         else
            write(*,'("Writing output plot: ",a30,/)') psfile
            psplot = 1
            go to 500
         end if
      end if

      end


C===================================================================
C Subroutines to compute derivatives for input to nonlinear fitting
C===================================================================

cccccccccccccccccccc
c
c     Solve for wavelength as a function of Z_fine and relative order
c     number (dN).  Order can vary, radius is fixed.
c
c     pars(1) and (2) = e and g
c     pars(3) = actual order (N0)
c
c     x is the argument to the function, but is really an index into the
c     arrays of z and dord.
c
      subroutine waveordfunc(x,pars,y,dydpar,npar)

      implicit none

      integer MAXLINES
      parameter(MAXLINES=200)
      real x,pars(npar),dydpar(npar),ztmp,dordtmp,ordtmp,y
      integer npar
      real wave(MAXLINES),z(MAXLINES),rad(MAXLINES),dord(MAXLINES)
      common /linedata/ wave,z,dord,rad

      ztmp = z(nint(x))
      dordtmp = dord(nint(x))
      ordtmp = pars(3)+dordtmp
      
      y = 2.0 / ordtmp * (pars(1) + pars(2)*ztmp)

      dydpar(1) = 2.0 / ordtmp
      dydpar(2) = 2.0 / ordtmp * ztmp
      dydpar(3) = 2.0 * (pars(1) + pars(2)*ztmp) * 
     +     -1.0/ordtmp**2
      dydpar(4) = 0.0

c      write(*,*) x,ztmp,dordtmp,radtmp,y
c      write(*,*) pars(1),pars(2),pars(3),pars(4),sqrt1rfsq

      return
      end

cccccccccccccccccccc
c
c     Solve for wavelength as function of Z_fine and radius.  Order is
c     fixed.
c
c     pars(1) and (2) = a and b
c     pars(4) = focal length
c
      subroutine waveradfunc(x,pars,y,dydpar,npar)

      implicit none

      integer MAXLINES
      parameter(MAXLINES=200)
      real x,pars(npar),dydpar(npar),ztmp,radtmp,fsq,rfsq,sqrt1rfsq,y
      integer npar
      real wave(MAXLINES),z(MAXLINES),rad(MAXLINES),dord(MAXLINES)
      common /linedata/ wave,z,dord,rad

      ztmp = z(nint(x))
c      dordtmp = dord(nint(x))
      radtmp = rad(nint(x))
      fsq = pars(4)**2
      rfsq = radtmp**2/fsq
      sqrt1rfsq = sqrt(1.0 + rfsq)
      
      y = (pars(1) + pars(2)*ztmp) / sqrt1rfsq

      dydpar(1) = 1.0 / sqrt1rfsq
      dydpar(2) = ztmp / sqrt1rfsq
      dydpar(3) = 0.0
      dydpar(4) = (pars(1) + pars(2)*ztmp) * 
     +     1.0/sqrt1rfsq/(1.0 + rfsq)*radtmp**2/pars(4)**3

      return
      end

cccccccccccccccccccc
c
c     Solve for wavelength as a function of Z_fine, relative order #,
c     and radius.
c
c     pars(1) and (2) = e and g
c     pars(3) = N0
c     pars(4) = focal length
c
      subroutine waveordradfunc(x,pars,y,dydpar,npar)

      implicit none

      integer MAXLINES
      parameter(MAXLINES=200)
      real x,pars(npar),dydpar(npar)
      real ztmp,dordtmp,ordtmp,radtmp,fsq,rfsq,sqrt1rfsq,y
      integer npar
      real wave(MAXLINES),z(MAXLINES),rad(MAXLINES),dord(MAXLINES)
      common /linedata/ wave,z,dord,rad

      ztmp = z(nint(x))
      dordtmp = dord(nint(x))
      ordtmp = pars(3)+dordtmp
      radtmp = rad(nint(x))
      fsq = pars(4)**2
      rfsq = radtmp**2/fsq
      sqrt1rfsq = sqrt(1.0 + rfsq)
      
      y = 2.0 / ordtmp * (pars(1) + pars(2)*ztmp) / sqrt1rfsq

      dydpar(1) = 2.0 / ordtmp / sqrt1rfsq
      dydpar(2) = ztmp * 2.0 / ordtmp / sqrt1rfsq
      dydpar(3) = 2.0 * (pars(1) + pars(2)*ztmp) / sqrt1rfsq *
     +     -1.0/ordtmp**2
      dydpar(4) = 2.0 / ordtmp * (pars(1) + pars(2)*ztmp) * 
     +     1.0/sqrt1rfsq/(1.0 + rfsq)*radtmp**2/pars(4)**3

c      write(*,*) x,ztmp,dordtmp,radtmp,y
c      write(*,*) pars(1),pars(2),pars(3),pars(4),sqrt1rfsq

      return
      end
